#!/usr/bin/perl

use strict;

my $fname;                    # file name of the file to be analysed
my $output = "";              # contains the contents of the new file
my $InsideFunction = 0;       # 1 if inside function (DECLARE)
my $InsideComment = 0;        # 1 if inside function (no interpretion needed)
my $ExecuteFound = 0;         # 1 if a execute inside a DECLARE function has been found (afterwards no parameters can be found)
my $FuncBody = "";            # function body, the function comment is written at the end of a function. so the output can be created only at the end of the function.
my $ParmCount = 0;            # parameter count of current funtion
my $doxygen = 0;              # 1 if the current comment line is a doxygen comment            
my $funcName;                 # name of the current function
my $firstfunctionline=0;      # The first function line is replaced by the parameter definition 
my $functionpardef= "";       # Function parameter definition line


if ($#ARGV != 0)
{
  die "Argument must contain filename $#ARGV"
}
else
{
  $fname=$ARGV[0];
}
open(my $in, $fname);

# read file line by line
while (my $line = <$in>) #<$in>)
{

  $InsideComment = 0;
 
  # fortran comment starts with C 
  if(uc($line) =~ /^(C+)/ )
  {
    $InsideComment = 3;
    
  }

  
  if($InsideComment == 0)
  {
    # interpret variables
    if($InsideFunction)
    {

      #recognise end of function
      if($line =~ /(\s*)END(\s*)/i)
      {

        #no execute command inside function?
        if($ExecuteFound == 0)
        {
          $output=$output.")\n";
        }
      
	$output=$output.$functionpardef."\n"; # fortran functions needs a parameter definition line behind the function header!	  
	   
        $output=$output.$FuncBody;
        
        $FuncBody = "";
        $ParmCount = 0;
        $InsideFunction = 0;
        $ExecuteFound = 0;
  
      }
      else # not end command
      {
        # remove first line of fortran functions (they need a additional parameter def. line 
	# and doxygen needs the original line count in the output, so 1 line is temporary removed)
	if( $firstfunctionline == 0)
	{ 
	    $FuncBody = $FuncBody.$line;
	}	
	
      }
    
      if($ExecuteFound == 0)
      {
        if($line =~ /(\s*)EXECUTE(\s*)/i)
        {
          $ExecuteFound = 1;
          $output=$output.")\n";
	
	  # fortran function without in or output parameters
	  if( $firstfunctionline == 1)
	  { 
	    $FuncBody = $FuncBody.$line;
	  }
	}	
      }
    
      $firstfunctionline = 0;
      
      #interpret variables
      if($ExecuteFound == 0)
      {
         my $pos = index($line,"=");
         
         if($pos != -1) 
         {
              my $param = substr($line,0,$pos);
              
              $param =~ s/^\s+//g;
              $param =~ s/\s+$//g ; #remove spaces (at the end)
              
              # if($IsCLanguage == 1)
	      # spaces inside the param name? => local variable e.g "static int z=0;"
	      if($param =~ s/\b(.*)(\s)(.*)//o)
	      {
	         $pos = -1;
	      }
	      elsif($param =~ s/\b(.*)(\()(.*)//o)
	      {
	           $pos = -1;	        
	      }
	      
	      if($pos != -1)
	      {
                # first variable?
                if($ParmCount == 0)
                {
                  $output=$output.$param;
		  
		  # parameter def. line for fortran programs
		  $functionpardef= "      INTEGER*4 ".$param; 
		
                }
                else
                {
                  $output=$output.", ".$param; # comma between last param and this param! 
		  $functionpardef=$functionpardef.", ".$param;
                }
            
                $ParmCount = $ParmCount + 1;
              }
         }
      }
    } # inside function end
    
    
    if($line =~ /(\s*)DECLARE(\s*)(\S*)(\s*)({*)(\s*)/i)
    {
      $funcName = $3;
      
      if($funcName =~ /(.*){/)
      {
        $funcName = $1;
      }
      
      $output=$output."    SUBROUTINE ".$funcName."(";
      $InsideFunction = 1;
      $firstfunctionline = 1;
      $functionpardef= "";
      $FuncBody = "";
      $ParmCount = 0;
      
    }
  } 
 
  if($InsideFunction == 0) 
  {
    $output=$output.$line;
  }
  elsif($InsideComment != 0)
  {
    $FuncBody=$FuncBody.$line;
  }
}
$output=$output."\n";
print $output;
close $in;
